@@ -61,9 +61,11 @@ R_LIB_VERSION_HEADER (r_bin);
 #define R_BIN_REQ_DLOPEN    0x200000
 #define R_BIN_REQ_EXPORTS   0x400000
 #define R_BIN_REQ_VERSIONINFO 0x800000
-#define R_BIN_REQ_PACKAGE     0x1000000
-#define R_BIN_REQ_HEADER   0x2000000
-#define R_BIN_REQ_LISTPLUGINS   0x4000000
+#define R_BIN_REQ_PACKAGE   0x1000000
+#define R_BIN_REQ_HEADER    0x2000000
+#define R_BIN_REQ_LISTPLUGINS 0x4000000
+#define R_BIN_REQ_RESOURCES 0x8000000
+#define R_BIN_REQ_INITFINI 0x10000000
 
 /* RBinSymbol->method_flags : */
 #define R_BIN_METH_CLASS 0x0000000000000001L
@@ -115,7 +117,8 @@ enum {
 	R_STRING_TYPE_DETECT = '?',
 	R_STRING_TYPE_ASCII = 'a',
 	R_STRING_TYPE_UTF8 = 'u',
-	R_STRING_TYPE_WIDE = 'w',
+	R_STRING_TYPE_WIDE = 'w', // utf16 / widechar string
+	R_STRING_TYPE_WIDE32 = 'W', // utf32
 	R_STRING_TYPE_BASE64 = 'b',
 };
 
@@ -172,6 +175,7 @@ typedef struct r_bin_info_t {
 	int has_crypto;
 	int has_nx;
 	int big_endian;
+	bool has_lit;
 	char *actual_checksum;
 	char *claimed_checksum;
 	int pe_overlay;
@@ -264,6 +268,7 @@ typedef struct r_bin_t {
 	RIOBind iob;
 	char *force;
 	int is_debugger;
+	bool want_dbginfo;
 	int filter; // symbol filtering
 	char strfilter; // string filtering
 	int strpurge; // purge false positive strings
@@ -272,6 +277,7 @@ typedef struct r_bin_t {
 	ut64 filter_rules;
 	bool demanglercmd;
 	bool verbose;
+	bool io_owned;
 } RBin;
 
 typedef struct r_bin_xtr_metadata_t {
@@ -352,7 +358,7 @@ typedef struct r_bin_plugin_t {
 	RList/*<RBinMem>*/* (*mem)(RBinFile *arch);
 	RList/*<RBinReloc>*/* (*patch_relocs)(RBin *bin);
 	void (*header)(RBinFile *arch);
-	char* (*signature)(RBinFile *arch);
+	char* (*signature)(RBinFile *arch, bool json);
 	int (*demangle_type)(const char *str);
 	struct r_bin_dbginfo_t *dbginfo;
 	struct r_bin_write_t *write;
@@ -410,6 +416,7 @@ typedef struct r_bin_symbol_t {
 	const char *forwarder;
 	const char *bind;
 	const char *type;
+  	const char *rtype;
 	/* only used by java */
 	const char *visibility_str;
 	// ----------------
@@ -422,6 +429,7 @@ typedef struct r_bin_symbol_t {
 	int bits;
 	/* see R_BIN_METH_* constants */
 	ut64 method_flags;
+	int dup_count;
 } RBinSymbol;
 
 typedef struct r_bin_import_t {
@@ -468,8 +476,10 @@ typedef struct r_bin_field_t {
 	int size;
 	ut32 visibility;
 	char *name;
+	char *type;
 	char *comment;
 	char *format;
+	ut64 flags;
 } RBinField;
 
 R_API RBinField *r_bin_field_new(ut64 paddr, ut64 vaddr, int size, const char *name, const char *comment, const char *format);
@@ -500,11 +510,15 @@ typedef struct r_bin_write_t {
 
 typedef int (*RBinGetOffset)(RBin *bin, int type, int idx);
 typedef const char *(*RBinGetName)(RBin *bin, int type, int idx);
+typedef RList *(*RBinGetSections)(RBin *bin);
+typedef RBinSection *(*RBinGetSectionAt)(RBin *bin, ut64 addr);
 
 typedef struct r_bin_bind_t {
 	RBin *bin;
 	RBinGetOffset get_offset;
 	RBinGetName get_name;
+	RBinGetSections get_sections;
+	RBinGetSectionAt get_vsect_at;
 	ut32 visibility;
 } RBinBind;
 
@@ -513,11 +527,11 @@ typedef struct r_bin_bind_t {
 /* bin.c */
 R_API void r_bin_load_filter(RBin *bin, ut64 rules);
 R_API int r_bin_load(RBin *bin, const char *file, ut64 baseaddr, ut64 loadaddr, int xtr_idx, int fd, int rawstr);
-R_API int r_bin_reload(RBin *bin, RIODesc *desc, ut64 baseaddr);
+R_API int r_bin_reload(RBin *bin, int fd, ut64 baseaddr);
 R_API int r_bin_load_as(RBin *bin, const char *file, ut64 baseaddr, ut64 loadaddr, int xtr_idx, int fd, int rawstr, int fileoffset, const char *name);
-R_API int r_bin_load_io(RBin *bin, RIODesc *desc, ut64 baseaddr, ut64 loadaddr, int xtr_idx);
-R_API bool r_bin_load_io_at_offset_as(RBin *bin, RIODesc *desc, ut64 baseaddr, ut64 loadaddr, int xtr_idx, ut64 offset, const char *name);
-R_API int r_bin_load_io_at_offset_as_sz(RBin *bin, RIODesc *desc, ut64 baseaddr, ut64 loadaddr, int xtr_idx, ut64 offset, const char *name, ut64 sz);
+R_API int r_bin_load_io(RBin *bin, int fd, ut64 baseaddr, ut64 loadaddr, int xtr_idx);
+R_API bool r_bin_load_io_at_offset_as(RBin *bin, int fd, ut64 baseaddr, ut64 loadaddr, int xtr_idx, ut64 offset, const char *name);
+R_API int r_bin_load_io_at_offset_as_sz(RBin *bin, int fd, ut64 baseaddr, ut64 loadaddr, int xtr_idx, ut64 offset, const char *name, ut64 sz);
 R_API void r_bin_bind(RBin *b, RBinBind *bnd);
 R_API int r_bin_add(RBin *bin, RBinPlugin *foo);
 R_API int r_bin_xtr_add(RBin *bin, RBinXtrPlugin *foo);
@@ -600,7 +614,7 @@ R_API void r_bin_iobind(RBin *bin, RIO *io);
 R_API RBinFile * r_bin_cur(RBin *bin);
 R_API RBinObject * r_bin_cur_object(RBin *bin);
 R_API int r_bin_file_set_cur_binfile_obj(RBin * bin, RBinFile *bf, RBinObject *obj);
-R_API int r_bin_io_load(RBin *bin, RIO *io, RIODesc *desc, ut64 baseaddr, ut64 loadaddr, int dummy);
+R_API int r_bin_io_load(RBin *bin, RIO *io, int fd, ut64 baseaddr, ut64 loadaddr, int dummy);
 
 R_API int r_bin_select(RBin *bin, const char *arch, int bits, const char *name);
 R_API int r_bin_select_idx(RBin *bin, const char *name, int idx);
@@ -638,7 +652,7 @@ R_API ut64 r_bin_wr_scn_resize(RBin *bin, const char *name, ut64 size);
 R_API bool r_bin_wr_scn_perms(RBin *bin, const char *name, int perms);
 R_API bool r_bin_wr_rpath_del(RBin *bin);
 R_API bool r_bin_wr_entry(RBin *bin, ut64 addr);
-R_API int r_bin_wr_output(RBin *bin, const char *filename);
+R_API bool r_bin_wr_output(RBin *bin, const char *filename);
 R_API int r_bin_dwarf_parse_info(RBinDwarfDebugAbbrev *da, RBin *a, int mode);
 R_API RList *r_bin_dwarf_parse_line(RBin *a, int mode);
 R_API RList *r_bin_dwarf_parse_aranges(RBin *a, int mode);
@@ -673,6 +687,7 @@ extern RBinPlugin r_bin_plugin_bf;
 extern RBinPlugin r_bin_plugin_te;
 extern RBinPlugin r_bin_plugin_mach0;
 extern RBinPlugin r_bin_plugin_mach064;
+extern RBinPlugin r_bin_plugin_mdmp;
 extern RBinPlugin r_bin_plugin_java;
 extern RBinPlugin r_bin_plugin_dex;
 extern RBinPlugin r_bin_plugin_dummy;
@@ -702,6 +717,7 @@ extern RBinPlugin r_bin_plugin_avr;
 extern RBinPlugin r_bin_plugin_menuet;
 extern RBinPlugin r_bin_plugin_wasm;
 extern RBinPlugin r_bin_plugin_nro;
+extern RBinPlugin r_bin_plugin_nso;
 extern RBinPlugin r_bin_plugin_sfc;
 
 #ifdef __cplusplus
