@@ -107,9 +107,6 @@
   /* Must be included before windows.h */
   #include <winsock2.h>
   #define WIN32_LEAN_AND_MEAN
-  #else
-  /* Deprecated */
-  //#include <winsock.h>
   #endif
   typedef int socklen_t;
   #undef USE_SOCKETS
@@ -402,7 +399,7 @@ static inline void *r_new_copy(int size, void *data) {
 #define R_SYS_ARCH "x86"
 #define R_SYS_BITS (R_SYS_BITS_32 | R_SYS_BITS_64)
 #define R_SYS_ENDIAN 0
-#elif __powerpc__
+#elif __POWERPC__
 # define R_SYS_ARCH "ppc"
 # ifdef __powerpc64__
 #  define R_SYS_BITS (R_SYS_BITS_32 | R_SYS_BITS_64)
